@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /* GIO - GLib Input, Output and Streaming Library
- * 
+ *
  * Copyright (C) 2006-2007 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -28,6 +28,7 @@
 
 #include <glib.h>
 #include <locale.h>
+#include <glib/gi18n.h>
 #include <gio/gio.h>
 
 #ifdef HAVE_TERMIOS_H
@@ -48,15 +49,15 @@ static gboolean mount_monitor = FALSE;
 static const char *unmount_scheme = NULL;
 static const char *mount_device_file = NULL;
 
-static const GOptionEntry entries[] = 
+static const GOptionEntry entries[] =
 {
-  { "mountable", 'm', 0, G_OPTION_ARG_NONE, &mount_mountable, "Mount as mountable", NULL },
-  { "device", 'd', 0, G_OPTION_ARG_STRING, &mount_device_file, "Mount volume with device file", NULL},
-  { "unmount", 'u', 0, G_OPTION_ARG_NONE, &mount_unmount, "Unmount", NULL},
-  { "unmount-scheme", 's', 0, G_OPTION_ARG_STRING, &unmount_scheme, "Unmount all mounts with the given scheme", NULL},
-  { "list", 'l', 0, G_OPTION_ARG_NONE, &mount_list, "List", NULL},
-  { "detail", 'i', 0, G_OPTION_ARG_NONE, &extra_detail, "Show extra information for List and Monitor", NULL},
-  { "monitor", 'o', 0, G_OPTION_ARG_NONE, &mount_monitor, "Monitor events", NULL},
+  { "mountable", 'm', 0, G_OPTION_ARG_NONE, &mount_mountable, N_("Mount as mountable"), NULL },
+  { "device", 'd', 0, G_OPTION_ARG_STRING, &mount_device_file, N_("Mount volume with device file"), NULL},
+  { "unmount", 'u', 0, G_OPTION_ARG_NONE, &mount_unmount, N_("Unmount"), NULL},
+  { "unmount-scheme", 's', 0, G_OPTION_ARG_STRING, &unmount_scheme, N_("Unmount all mounts with the given scheme"), NULL},
+  { "list", 'l', 0, G_OPTION_ARG_NONE, &mount_list, N_("List"), NULL},
+  { "detail", 'i', 0, G_OPTION_ARG_NONE, &extra_detail, N_("Show extra information for List and Monitor"), NULL},
+  { "monitor", 'o', 0, G_OPTION_ARG_NONE, &mount_monitor, N_("Monitor events"), NULL},
   { NULL }
 };
 
@@ -64,7 +65,7 @@ static char *
 prompt_for (const char *prompt, const char *default_value, gboolean echo)
 {
 #ifdef HAVE_TERMIOS_H
-  struct termios term_attr; 
+  struct termios term_attr;
   int old_flags;
   gboolean restore_flags;
 #endif
@@ -82,10 +83,10 @@ prompt_for (const char *prompt, const char *default_value, gboolean echo)
   restore_flags = FALSE;
   if (!echo && tcgetattr (STDIN_FILENO, &term_attr) == 0)
     {
-      old_flags = term_attr.c_lflag; 
+      old_flags = term_attr.c_lflag;
       term_attr.c_lflag &= ~ECHO;
       restore_flags = TRUE;
-      
+
       if (tcsetattr (STDIN_FILENO, TCSAFLUSH, &term_attr) != 0)
         g_print ("Warning! Password will be echoed");
     }
@@ -93,7 +94,7 @@ prompt_for (const char *prompt, const char *default_value, gboolean echo)
 #endif
 
   fgets(data, sizeof (data), stdin);
-  
+
 #ifdef HAVE_TERMIOS_H
   if (restore_flags)
     {
@@ -108,7 +109,7 @@ prompt_for (const char *prompt, const char *default_value, gboolean echo)
 
   if (!echo)
     g_print ("\n");
-  
+
   if (*data == 0 && default_value)
     return g_strdup (default_value);
   return g_strdup (data);
@@ -116,10 +117,10 @@ prompt_for (const char *prompt, const char *default_value, gboolean echo)
 
 static void
 ask_password_cb (GMountOperation *op,
-		 const char      *message,
-		 const char      *default_user,
-		 const char      *default_domain,
-		 GAskPasswordFlags flags)
+                 const char      *message,
+                 const char      *default_user,
+                 const char      *default_domain,
+                 GAskPasswordFlags flags)
 {
   char *s;
   g_print ("%s\n", message);
@@ -130,14 +131,14 @@ ask_password_cb (GMountOperation *op,
       g_mount_operation_set_username (op, s);
       g_free (s);
     }
-  
+
   if (flags & G_ASK_PASSWORD_NEED_DOMAIN)
     {
       s = prompt_for ("Domain", default_domain, TRUE);
       g_mount_operation_set_domain (op, s);
       g_free (s);
     }
-  
+
   if (flags & G_ASK_PASSWORD_NEED_PASSWORD)
     {
       s = prompt_for ("Password", NULL, FALSE);
@@ -150,29 +151,29 @@ ask_password_cb (GMountOperation *op,
 
 static void
 mount_mountable_done_cb (GObject *object,
-			 GAsyncResult *res,
-			 gpointer user_data)
+                         GAsyncResult *res,
+                         gpointer user_data)
 {
   GFile *target;
   GError *error = NULL;
-  
+
   target = g_file_mount_mountable_finish (G_FILE (object), res, &error);
 
   if (target == NULL)
-    g_print ("Error mounting location: %s\n", error->message);
+    g_printerr (_("Error mounting location: %s\n"), error->message);
   else
     g_object_unref (target);
-  
+
   outstanding_mounts--;
-  
+
   if (outstanding_mounts == 0)
     g_main_loop_quit (main_loop);
 }
 
 static void
 mount_done_cb (GObject *object,
-	       GAsyncResult *res,
-	       gpointer user_data)
+               GAsyncResult *res,
+               gpointer user_data)
 {
   gboolean succeeded;
   GError *error = NULL;
@@ -180,10 +181,10 @@ mount_done_cb (GObject *object,
   succeeded = g_file_mount_enclosing_volume_finish (G_FILE (object), res, &error);
 
   if (!succeeded)
-    g_print ("Error mounting location: %s\n", error->message);
-  
+    g_printerr (_("Error mounting location: %s\n"), error->message);
+
   outstanding_mounts--;
-  
+
   if (outstanding_mounts == 0)
     g_main_loop_quit (main_loop);
 }
@@ -192,7 +193,7 @@ static GMountOperation *
 new_mount_op (void)
 {
   GMountOperation *op;
-  
+
   op = g_mount_operation_new ();
 
   g_signal_connect (op, "ask_password", G_CALLBACK (ask_password_cb), NULL);
@@ -220,7 +221,7 @@ mount (GFile *file)
     g_file_mount_mountable (file, 0, op, NULL, mount_mountable_done_cb, op);
   else
     g_file_mount_enclosing_volume (file, 0, op, NULL, mount_done_cb, op);
-    
+
   outstanding_mounts++;
 }
 
@@ -232,15 +233,15 @@ unmount_done_cb (GObject *object,
   gboolean succeeded;
   GError *error = NULL;
 
-  succeeded = g_mount_unmount_finish (G_MOUNT (object), res, &error);
+  succeeded = g_mount_unmount_with_operation_finish (G_MOUNT (object), res, &error);
 
   g_object_unref (G_MOUNT (object));
 
   if (!succeeded)
-    g_print ("Error unmounting mount: %s\n", error->message);
-  
+    g_printerr (_("Error unmounting mount: %s\n"), error->message);
+
   outstanding_mounts--;
-  
+
   if (outstanding_mounts == 0)
     g_main_loop_quit (main_loop);
 }
@@ -250,6 +251,7 @@ unmount (GFile *file)
 {
   GMount *mount;
   GError *error = NULL;
+  GMountOperation *mount_op;
 
   if (file == NULL)
     return;
@@ -257,11 +259,13 @@ unmount (GFile *file)
   mount = g_file_find_enclosing_mount (file, NULL, &error);
   if (mount == NULL)
     {
-      g_print ("Error finding enclosing mount: %s\n", error->message);
+      g_printerr (_("Error finding enclosing mount: %s\n"), error->message);
       return;
     }
 
-  g_mount_unmount (mount, 0, NULL, unmount_done_cb, NULL);
+  mount_op = new_mount_op ();
+  g_mount_unmount_with_operation (mount, 0, mount_op, NULL, unmount_done_cb, NULL);
+  g_object_unref (mount_op);
 
   outstanding_mounts++;
 }
@@ -278,7 +282,7 @@ iterate_gmain_timeout_function (gpointer data)
 static void
 iterate_gmain(void)
 {
-  g_timeout_add (500, iterate_gmain_timeout_function, NULL);  
+  g_timeout_add (500, iterate_gmain_timeout_function, NULL);
   g_main_loop_run (main_loop);
 }
 
@@ -347,8 +351,8 @@ get_type_name (gpointer object)
 
 static void
 list_mounts (GList *mounts,
-	     int indent,
-	     gboolean only_with_no_volume)
+             int indent,
+             gboolean only_with_no_volume)
 {
   GList *l;
   int c;
@@ -359,25 +363,25 @@ list_mounts (GList *mounts,
   GIcon *icon;
   char **x_content_types;
   char *type_name;
-  
+
   for (c = 0, l = mounts; l != NULL; l = l->next, c++)
     {
       mount = (GMount *) l->data;
-      
+
       if (only_with_no_volume)
-	{
-	  volume = g_mount_get_volume (mount);
-	  if (volume != NULL)
-	    {
+        {
+          volume = g_mount_get_volume (mount);
+          if (volume != NULL)
+            {
               g_object_unref (volume);
-	      continue;
-	    }
-	}
+              continue;
+            }
+        }
 
       name = g_mount_get_name (mount);
       root = g_mount_get_root (mount);
       uri = g_file_get_uri (root);
-      
+
       g_print ("%*sMount(%d): %s -> %s\n", indent, "", c, name, uri);
 
       type_name = get_type_name (mount);
@@ -385,17 +389,17 @@ list_mounts (GList *mounts,
       g_free (type_name);
 
       if (extra_detail)
-	{
-	  uuid = g_mount_get_uuid (mount);
-	  if (uuid)
-	    g_print ("%*suuid=%s\n", indent + 2, "", uuid);
+        {
+          uuid = g_mount_get_uuid (mount);
+          if (uuid)
+            g_print ("%*suuid=%s\n", indent + 2, "", uuid);
 
           icon = g_mount_get_icon (mount);
           if (icon)
             {
               if (G_IS_THEMED_ICON (icon))
                 show_themed_icon_names (G_THEMED_ICON (icon), indent + 2);
-              
+
               g_object_unref (icon);
             }
 
@@ -410,22 +414,22 @@ list_mounts (GList *mounts,
             }
           g_strfreev (x_content_types);
 
-	  g_print ("%*scan_unmount=%d\n", indent + 2, "", g_mount_can_unmount (mount));
-	  g_print ("%*scan_eject=%d\n", indent + 2, "", g_mount_can_eject (mount));
-	  g_print ("%*sis_shadowed=%d\n", indent + 2, "", g_mount_is_shadowed (mount));
-	  g_free (uuid);
-	}
-      
+          g_print ("%*scan_unmount=%d\n", indent + 2, "", g_mount_can_unmount (mount));
+          g_print ("%*scan_eject=%d\n", indent + 2, "", g_mount_can_eject (mount));
+          g_print ("%*sis_shadowed=%d\n", indent + 2, "", g_mount_is_shadowed (mount));
+          g_free (uuid);
+        }
+
       g_object_unref (root);
       g_free (name);
       g_free (uri);
-    }  
+    }
 }
 
 static void
 list_volumes (GList *volumes,
-	      int indent,
-	      gboolean only_with_no_drive)
+              int indent,
+              gboolean only_with_no_drive)
 {
   GList *l, *mounts;
   int c, i;
@@ -438,23 +442,23 @@ list_volumes (GList *volumes,
   char **ids;
   GIcon *icon;
   char *type_name;
-  
+
   for (c = 0, l = volumes; l != NULL; l = l->next, c++)
     {
       volume = (GVolume *) l->data;
-      
+
       if (only_with_no_drive)
-	{
-	  drive = g_volume_get_drive (volume);
-	  if (drive != NULL)
-	    {
-	      g_object_unref (drive);
-	      continue;
-	    }
-	}
-      
+        {
+          drive = g_volume_get_drive (volume);
+          if (drive != NULL)
+            {
+              g_object_unref (drive);
+              continue;
+            }
+        }
+
       name = g_volume_get_name (volume);
-      
+
       g_print ("%*sVolume(%d): %s\n", indent, "", c, name);
       g_free (name);
 
@@ -463,26 +467,26 @@ list_volumes (GList *volumes,
       g_free (type_name);
 
       if (extra_detail)
-	{
-	  ids = g_volume_enumerate_identifiers (volume);
-	  if (ids && ids[0] != NULL)
-	    {
-	      g_print ("%*sids:\n", indent+2, "");
-	      for (i = 0; ids[i] != NULL; i++)
-		{
-		  char *id = g_volume_get_identifier (volume,
-						      ids[i]);
-		  g_print ("%*s %s: '%s'\n", indent+2, "", ids[i], id);
-		  g_free (id);
-		}
-	    }
-	  g_strfreev (ids);
-	  
-	  uuid = g_volume_get_uuid (volume);
-	  if (uuid)
-	    g_print ("%*suuid=%s\n", indent + 2, "", uuid);
-	  activation_root = g_volume_get_activation_root (volume);
-	  if (activation_root)
+        {
+          ids = g_volume_enumerate_identifiers (volume);
+          if (ids && ids[0] != NULL)
+            {
+              g_print ("%*sids:\n", indent+2, "");
+              for (i = 0; ids[i] != NULL; i++)
+                {
+                  char *id = g_volume_get_identifier (volume,
+                                                      ids[i]);
+                  g_print ("%*s %s: '%s'\n", indent+2, "", ids[i], id);
+                  g_free (id);
+                }
+            }
+          g_strfreev (ids);
+
+          uuid = g_volume_get_uuid (volume);
+          if (uuid)
+            g_print ("%*suuid=%s\n", indent + 2, "", uuid);
+          activation_root = g_volume_get_activation_root (volume);
+          if (activation_root)
             {
               char *uri;
               uri = g_file_get_uri (activation_root);
@@ -490,34 +494,35 @@ list_volumes (GList *volumes,
               g_free (uri);
               g_object_unref (activation_root);
             }
-      icon = g_volume_get_icon (volume);
-      if (icon)
-        {
-          if (G_IS_THEMED_ICON (icon))
-            show_themed_icon_names (G_THEMED_ICON (icon), indent + 2);
+          icon = g_volume_get_icon (volume);
+          if (icon)
+            {
+              if (G_IS_THEMED_ICON (icon))
+                show_themed_icon_names (G_THEMED_ICON (icon), indent + 2);
 
-          g_object_unref (icon);
+              g_object_unref (icon);
+            }
+
+          g_print ("%*scan_mount=%d\n", indent + 2, "", g_volume_can_mount (volume));
+          g_print ("%*scan_eject=%d\n", indent + 2, "", g_volume_can_eject (volume));
+          g_print ("%*sshould_automount=%d\n", indent + 2, "", g_volume_should_automount (volume));
+          g_free (uuid);
         }
 
-	  g_print ("%*scan_mount=%d\n", indent + 2, "", g_volume_can_mount (volume));
-	  g_print ("%*scan_eject=%d\n", indent + 2, "", g_volume_can_eject (volume));
-	  g_free (uuid);
-	}
-      
       mount = g_volume_get_mount (volume);
       if (mount)
-	{
-	  mounts = g_list_prepend (NULL, mount);
-	  list_mounts (mounts, indent + 2, FALSE);
-	  g_list_free (mounts);
-	  g_object_unref (mount);
-	}
-    }  
+        {
+          mounts = g_list_prepend (NULL, mount);
+          list_mounts (mounts, indent + 2, FALSE);
+          g_list_free (mounts);
+          g_object_unref (mount);
+        }
+    }
 }
 
 static void
 list_drives (GList *drives,
-	     int indent)
+             int indent)
 {
   GList *volumes, *l;
   int c, i;
@@ -526,34 +531,37 @@ list_drives (GList *drives,
   char **ids;
   GIcon *icon;
   char *type_name;
-  
+
   for (c = 0, l = drives; l != NULL; l = l->next, c++)
     {
       drive = (GDrive *) l->data;
       name = g_drive_get_name (drive);
-      
+
       g_print ("%*sDrive(%d): %s\n", indent, "", c, name);
       g_free (name);
 
       type_name = get_type_name (drive);
       g_print ("%*sType: %s\n", indent+2, "", type_name);
       g_free (type_name);
-      
+
       if (extra_detail)
-	{
-	  ids = g_drive_enumerate_identifiers (drive);
-	  if (ids && ids[0] != NULL)
-	    {
-	      g_print ("%*sids:\n", indent+2, "");
-	      for (i = 0; ids[i] != NULL; i++)
-		{
-		  char *id = g_drive_get_identifier (drive,
-						     ids[i]);
-		  g_print ("%*s %s: '%s'\n", indent+2, "", ids[i], id);
-		  g_free (id);
-		}
-	    }
-	  g_strfreev (ids);
+        {
+          GEnumValue *enum_value;
+          gpointer klass;
+
+          ids = g_drive_enumerate_identifiers (drive);
+          if (ids && ids[0] != NULL)
+            {
+              g_print ("%*sids:\n", indent+2, "");
+              for (i = 0; ids[i] != NULL; i++)
+                {
+                  char *id = g_drive_get_identifier (drive,
+                                                     ids[i]);
+                  g_print ("%*s %s: '%s'\n", indent+2, "", ids[i], id);
+                  g_free (id);
+                }
+            }
+          g_strfreev (ids);
 
           icon = g_drive_get_icon (drive);
           if (icon)
@@ -563,13 +571,24 @@ list_drives (GList *drives,
                   g_object_unref (icon);
           }
 
-	  g_print ("%*sis_media_removable=%d\n", indent + 2, "", g_drive_is_media_removable (drive));
-	  g_print ("%*shas_media=%d\n", indent + 2, "", g_drive_has_media (drive));
-	  g_print ("%*sis_media_check_automatic=%d\n", indent + 2, "", g_drive_is_media_check_automatic (drive));
-	  g_print ("%*scan_poll_for_media=%d\n", indent + 2, "", g_drive_can_poll_for_media (drive));
-	  g_print ("%*scan_eject=%d\n", indent + 2, "", g_drive_can_eject (drive));
-	}
-      
+          g_print ("%*sis_media_removable=%d\n", indent + 2, "", g_drive_is_media_removable (drive));
+          g_print ("%*shas_media=%d\n", indent + 2, "", g_drive_has_media (drive));
+          g_print ("%*sis_media_check_automatic=%d\n", indent + 2, "", g_drive_is_media_check_automatic (drive));
+          g_print ("%*scan_poll_for_media=%d\n", indent + 2, "", g_drive_can_poll_for_media (drive));
+          g_print ("%*scan_eject=%d\n", indent + 2, "", g_drive_can_eject (drive));
+          g_print ("%*scan_start=%d\n", indent + 2, "", g_drive_can_start (drive));
+          g_print ("%*scan_stop=%d\n", indent + 2, "", g_drive_can_stop (drive));
+
+          enum_value = NULL;
+          klass = g_type_class_ref (G_TYPE_DRIVE_START_STOP_TYPE);
+          if (klass != NULL)
+            {
+              enum_value = g_enum_get_value (klass, g_drive_get_start_stop_type (drive));
+              g_print ("%*sstart_stop_type=%s\n", indent + 2, "",
+                       enum_value != NULL ? enum_value->value_nick : "UNKNOWN");
+              g_type_class_unref (klass);
+            }
+        }
       volumes = g_drive_get_volumes (drive);
       list_volumes (volumes, indent + 2, FALSE);
       g_list_foreach (volumes, (GFunc)g_object_unref, NULL);
@@ -585,7 +604,7 @@ list_monitor_items(void)
   GList *drives, *volumes, *mounts;
 
   volume_monitor = g_volume_monitor_get();
-  
+
   /* populate gvfs network mounts */
   iterate_gmain();
 
@@ -593,12 +612,12 @@ list_monitor_items(void)
   list_drives (drives, 0);
   g_list_foreach (drives, (GFunc)g_object_unref, NULL);
   g_list_free (drives);
-  
+
   volumes = g_volume_monitor_get_volumes (volume_monitor);
   list_volumes (volumes, 0, TRUE);
   g_list_foreach (volumes, (GFunc)g_object_unref, NULL);
   g_list_free (volumes);
-  
+
   mounts = g_volume_monitor_get_mounts (volume_monitor);
   list_mounts (mounts, 0, TRUE);
   g_list_foreach (mounts, (GFunc)g_object_unref, NULL);
@@ -651,9 +670,9 @@ mount_with_device_file_cb (GObject *object,
 
   if (!succeeded)
     {
-      g_print ("Error mounting %s: %s\n",
-               g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE),
-               error->message);
+      g_printerr (_("Error mounting %s: %s\n"),
+                  g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE),
+                  error->message);
     }
   else
     {
@@ -665,7 +684,7 @@ mount_with_device_file_cb (GObject *object,
       root = g_mount_get_root (mount);
       mount_path = g_file_get_path (root);
 
-      g_print ("Mounted %s at %s\n",
+      g_print (_("Mounted %s at %s\n"),
                g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE),
                mount_path);
 
@@ -716,7 +735,7 @@ mount_with_device_file (const char *device_file)
 
   if (outstanding_mounts == 0)
     {
-      g_print ("No volume for device file %s\n", device_file);
+      g_print (_("No volume for device file %s\n"), device_file);
       return;
     }
 
@@ -874,10 +893,8 @@ monitor_drive_eject_button (GVolumeMonitor *volume_monitor, GDrive *drive)
 static void
 monitor (void)
 {
-  GMainLoop *loop;
   GVolumeMonitor *volume_monitor;
 
-  loop = g_main_loop_new (NULL, FALSE);
   volume_monitor = g_volume_monitor_get ();
 
   g_signal_connect (volume_monitor, "mount-added", (GCallback) monitor_mount_added, NULL);
@@ -903,19 +920,19 @@ main (int argc, char *argv[])
   GOptionContext *context;
   GError *error;
   GFile *file;
-  
+
   setlocale (LC_ALL, "");
 
   g_type_init ();
-  
+
   error = NULL;
-  context = g_option_context_new ("- mount <location>");
+  context = g_option_context_new (_("- mount <location>"));
   g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
   g_option_context_parse (context, &argc, &argv, &error);
   g_option_context_free (context);
 
   main_loop = g_main_loop_new (NULL, FALSE);
-  
+
   if (mount_list)
     list_monitor_items ();
   else if (mount_device_file != NULL)
@@ -933,19 +950,19 @@ main (int argc, char *argv[])
   else if (argc > 1)
     {
       int i;
-      
+
       for (i = 1; i < argc; i++) {
-	file = g_file_new_for_commandline_arg (argv[i]);
-	if (mount_unmount)
-	  unmount (file);
-	else
-	  mount (file);
-	g_object_unref (file);
+        file = g_file_new_for_commandline_arg (argv[i]);
+        if (mount_unmount)
+          unmount (file);
+        else
+          mount (file);
+        g_object_unref (file);
       }
     }
-  
+
   if (outstanding_mounts > 0)
     g_main_loop_run (main_loop);
-  
+
   return 0;
 }
